@@ -1,41 +1,60 @@
 //! # Relocation computations
+//!
+//! The following notation is used to describe relocation computations
+//! specific to x86_64 ELF.
+//!
+//!  * A: The addend used to compute the value of the relocatable field.
+//!  * B: The base address at which a shared object is loaded into memory
+//!       during execution. Generally, a shared object file is built with a
+//!       base virtual address of 0. However, the execution address of the
+//!       shared object is different.
+//!  * G: The offset into the global offset table at which the address of
+//!       the relocation entry's symbol resides during execution.
+//!  * GOT: The address of the global offset table.
+//!  * L: The section offset or address of the procedure linkage table entry
+//!       for a symbol.
+//!  * P: The section offset or address of the storage unit being relocated,
+//!       computed using r_offset.
+//!  * S: The value of the symbol whose index resides in the relocation entry.
+//!  * Z: The size of the symbol whose index resides in the relocation entry.
+//!
 //! Below are some common x86_64 relocation computations you might find useful:
 //!
-//! | Relocation | Value | Size | Formula |
-//! |:-----------|:------|:-----|:-------|
-//! | R_X86_64_NONE | 0 | none | none |
-//! | R_X86_64_64 | 1 | word64 | S + A |
-//! | R_X86_64_PC32 | 2 | word32 | S + A - P |
-//! | R_X86_64_GOT32 | 3 | word32 | G + A |
-//! | R_X86_64_PLT32 | 4 | word32 | L + A - P |
-//! | R_X86_64_COPY | 5 | none | none |
-//! | R_X86_64_GLOB_DAT | 6 | word64 | S |
-//! | R_X86_64_JUMP_SLOT | 7 | word64 | S |
-//! | R_X86_64_RELATIVE | 8 | word64 | B + A |
-//! | R_X86_64_GOTPCREL | 9 | word32 | G + GOT + A - P |
-//! | R_X86_64_32 | 10 | word32 | S + A |
-//! | R_X86_64_32S | 11 | word32 | S + A |
-//! | R_X86_64_16 | 12 | word16 | S + A |
-//! | R_X86_64_PC16 | 13 | word16 | S + A - P |
-//! | R_X86_64_8 | 14 | word8 | S + A |
-//! | R_X86_64_PC8 | 15 | word8 | S + A - P |
-//! | R_X86_64_DTPMOD64 | 16 | word64 | |
-//! | R_X86_64_DTPOFF64 | 17 | word64 | |
-//! | R_X86_64_TPOFF64 | 18 | word64 | |
-//! | R_X86_64_TLSGD | 19 | word32 | |
-//! | R_X86_64_TLSLD | 20 | word32 | |
-//! | R_X86_64_DTPOFF32 | 21 | word32 | |
-//! | R_X86_64_GOTTPOFF | 22 | word32 | |
-//! | R_X86_64_TPOFF32 | 23 | word32 | |
-//! | R_X86_64_PC64 | 24 | word64 | S + A - P |
-//! | R_X86_64_GOTOFF64 | 25 | word64 | S + A - GOT |
-//! | R_X86_64_GOTPC32 | 26 | word32 | GOT + A - P |
-//! | R_X86_64_SIZE32 | 32 | word32 | Z + A |
-//! | R_X86_64_SIZE64 | 33 | word64 | Z + A |
-//! | R_X86_64_GOTPC32_TLSDESC | 34 | word32 | |
-//! | R_X86_64_TLSDESC_CALL | 35 | none| |
-//! | R_X86_64_TLSDESC | 36 | word64×2 | |
-//! | R_X86_64_IRELATIVE | 37 | word64 | indirect (B + A) |
+//! | Relocation                | Value | Size      | Formula           |
+//! |:--------------------------|:------|:----------|:------------------|
+//! | `R_X86_64_NONE`           | 0     | NONE      | NONE              |
+//! | `R_X86_64_64`             | 1     | 64        | S + A             |
+//! | `R_X86_64_PC32`           | 2     | 32        | S + A - P         |
+//! | `R_X86_64_GOT32`          | 3     | 32        | G + A             |
+//! | `R_X86_64_PLT32`          | 4     | 32        | L + A - P         |
+//! | `R_X86_64_COPY`           | 5     | NONE      | NONE              |
+//! | `R_X86_64_GLOB_DAT`       | 6     | 64        | S                 |
+//! | `R_X86_64_JUMP_SLOT`      | 7     | 64        | S                 |
+//! | `R_X86_64_RELATIVE`       | 8     | 64        | B + A             |
+//! | `R_X86_64_GOTPCREL`       | 9     | 32        | G + GOT + A - P   |
+//! | `R_X86_64_32`             | 10    | 32        | S + A             |
+//! | `R_X86_64_32S`            | 11    | 32        | S + A             |
+//! | `R_X86_64_16`             | 12    | 16        | S + A             |
+//! | `R_X86_64_PC16`           | 13    | 16        | S + A - P         |
+//! | `R_X86_64_8`              | 14    | 8         | S + A             |
+//! | `R_X86_64_PC8`            | 15    | 8         | S + A - P         |
+//! | `R_X86_64_DTPMOD64`       | 16    | 64        |                   |
+//! | `R_X86_64_DTPOFF64`       | 17    | 64        |                   |
+//! | `R_X86_64_TPOFF64`        | 18    | 64        |                   |
+//! | `R_X86_64_TLSGD`          | 19    | 32        |                   |
+//! | `R_X86_64_TLSLD`          | 20    | 32        |                   |
+//! | `R_X86_64_DTPOFF32`       | 21    | 32        |                   |
+//! | `R_X86_64_GOTTPOFF`       | 22    | 32        |                   |
+//! | `R_X86_64_TPOFF32`        | 23    | 32        |                   |
+//! | `R_X86_64_PC64`           | 24    | 64        | S + A - P         |
+//! | `R_X86_64_GOTOFF64`       | 25    | 64        | S + A - GOT       |
+//! | `R_X86_64_GOTPC32`        | 26    | 32        | GOT + A - P       |
+//! | `R_X86_64_SIZE32`         | 32    | 32        | Z + A             |
+//! | `R_X86_64_SIZE64`         | 33    | 64        | Z + A             |
+//! | `R_X86_64_GOTPC32_TLSDESC`  34    | 32        |                   |
+//! | `R_X86_64_TLSDESC_CALL`   | 35    | NONE      |                   |
+//! | `R_X86_64_TLSDESC`        | 36    | 64 × 2    |                   |
+//! | `R_X86_64_IRELATIVE`      | 37    | 64        | indirect (B + A)  |
 //!
 //! TLS information is at http://people.redhat.com/aoliva/writeups/TLS/RFC-TLSDESC-x86.txt
 //!
@@ -43,12 +62,16 @@
 //! the value used in this relocation is the program address returned by the function,
 //! which takes no arguments, at the address of the result of the corresponding
 //! `R_X86_64_RELATIVE` relocation.
+//!
+//! Read more https://docs.oracle.com/cd/E23824_01/html/819-0690/chapter6-54839.html
 
 include!("constants_relocation.rs");
 
 macro_rules! elf_reloc {
     ($size:ident, $isize:ty) => {
         use core::fmt;
+        #[cfg(feature = "alloc")]
+        use scroll::{Pread, Pwrite, SizeWith};
         #[repr(C)]
         #[derive(Clone, Copy, PartialEq, Default)]
         #[cfg_attr(feature = "alloc", derive(Pread, Pwrite, SizeWith))]
@@ -79,38 +102,39 @@ macro_rules! elf_reloc {
             fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
                 let sym = r_sym(self.r_info);
                 let typ = r_type(self.r_info);
-                write!(f,
-                       "r_offset: {:x} r_typ: {} r_sym: {} r_addend: {:x}",
-                       self.r_offset,
-                       typ,
-                       sym,
-                       self.r_addend)
+                f.debug_struct("Rela")
+                    .field("r_offset", &format_args!("{:x}", self.r_offset))
+                    .field("r_info", &format_args!("{:x}", self.r_info))
+                    .field("r_addend", &format_args!("{:x}", self.r_addend))
+                    .field("r_typ", &typ)
+                    .field("r_sym", &sym)
+                    .finish()
             }
         }
         impl fmt::Debug for Rel {
             fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
                 let sym = r_sym(self.r_info);
                 let typ = r_type(self.r_info);
-                write!(f,
-                       "r_offset: {:x} r_typ: {} r_sym: {}",
-                       self.r_offset,
-                       typ,
-                       sym
-                )
+                f.debug_struct("Rel")
+                    .field("r_offset", &format_args!("{:x}", self.r_offset))
+                    .field("r_info", &format_args!("{:x}", self.r_info))
+                    .field("r_typ", &typ)
+                    .field("r_sym", &sym)
+                    .finish()
             }
         }
     };
 }
 
-macro_rules! elf_rela_std_impl { ($size:ident, $isize:ty) => {
-
-    if_alloc! {
-            use elf::reloc::Reloc;
+macro_rules! elf_rela_std_impl {
+    ($size:ident, $isize:ty) => {
+        if_alloc! {
+            use crate::elf::reloc::Reloc;
 
             use core::slice;
 
             if_std! {
-                use error::Result;
+                use crate::error::Result;
 
                 use std::fs::File;
                 use std::io::{Read, Seek};
@@ -120,8 +144,8 @@ macro_rules! elf_rela_std_impl { ($size:ident, $isize:ty) => {
             impl From<Rela> for Reloc {
                 fn from(rela: Rela) -> Self {
                     Reloc {
-                        r_offset: rela.r_offset as u64,
-                        r_addend: Some(rela.r_addend as i64),
+                        r_offset: u64::from(rela.r_offset),
+                        r_addend: Some(i64::from(rela.r_addend)),
                         r_sym: r_sym(rela.r_info) as usize,
                         r_type: r_type(rela.r_info),
                     }
@@ -131,7 +155,7 @@ macro_rules! elf_rela_std_impl { ($size:ident, $isize:ty) => {
             impl From<Rel> for Reloc {
                 fn from(rel: Rel) -> Self {
                     Reloc {
-                        r_offset: rel.r_offset as u64,
+                        r_offset: u64::from(rel.r_offset),
                         r_addend: None,
                         r_sym: r_sym(rel.r_info) as usize,
                         r_type: r_type(rel.r_info),
@@ -141,7 +165,7 @@ macro_rules! elf_rela_std_impl { ($size:ident, $isize:ty) => {
 
             impl From<Reloc> for Rela {
                 fn from(rela: Reloc) -> Self {
-                    let r_info = r_info(rela.r_sym as $size, rela.r_type as $size);
+                    let r_info = r_info(rela.r_sym as $size, $size::from(rela.r_type));
                     Rela {
                         r_offset: rela.r_offset as $size,
                         r_info: r_info,
@@ -152,7 +176,7 @@ macro_rules! elf_rela_std_impl { ($size:ident, $isize:ty) => {
 
             impl From<Reloc> for Rel {
                 fn from(rel: Reloc) -> Self {
-                    let r_info = r_info(rel.r_sym as $size, rel.r_type as $size);
+                    let r_info = r_info(rel.r_sym as $size, $size::from(rel.r_type));
                     Rel {
                         r_offset: rel.r_offset as $size,
                         r_info: r_info,
@@ -186,7 +210,7 @@ macro_rules! elf_rela_std_impl { ($size:ident, $isize:ty) => {
                 let mut relocs = vec![Rela::default(); count];
                 fd.seek(Start(offset as u64))?;
                 unsafe {
-                    fd.read(plain::as_mut_bytes(&mut *relocs))?;
+                    fd.read_exact(plain::as_mut_bytes(&mut *relocs))?;
                 }
                 Ok(relocs)
             }
@@ -194,10 +218,9 @@ macro_rules! elf_rela_std_impl { ($size:ident, $isize:ty) => {
     };
 }
 
-
 pub mod reloc32 {
 
-    pub use elf::reloc::*;
+    pub use crate::elf::reloc::*;
 
     elf_reloc!(u32, i32);
 
@@ -222,9 +245,8 @@ pub mod reloc32 {
     elf_rela_std_impl!(u32, i32);
 }
 
-
 pub mod reloc64 {
-    pub use elf::reloc::*;
+    pub use crate::elf::reloc::*;
 
     elf_reloc!(u64, i64);
 
@@ -238,7 +260,7 @@ pub mod reloc64 {
 
     #[inline(always)]
     pub fn r_type(info: u64) -> u32 {
-        (info & 0xffffffff) as u32
+        (info & 0xffff_ffff) as u32
     }
 
     #[inline(always)]
@@ -257,7 +279,7 @@ if_alloc! {
     use scroll::ctx::SizeWith;
     use core::fmt;
     use core::result;
-    use container::{Ctx, Container};
+    use crate::container::{Ctx, Container};
     #[cfg(feature = "endian_fd")]
     use alloc::vec::Vec;
 
@@ -284,8 +306,7 @@ if_alloc! {
     type RelocCtx = (bool, Ctx);
 
     impl ctx::SizeWith<RelocCtx> for Reloc {
-        type Units = usize;
-        fn size_with( &(is_rela, Ctx { container, .. }): &RelocCtx) -> Self::Units {
+        fn size_with( &(is_rela, Ctx { container, .. }): &RelocCtx) -> usize {
             match container {
                 Container::Little => {
                     if is_rela { reloc32::SIZEOF_RELA } else { reloc32::SIZEOF_REL }
@@ -298,9 +319,8 @@ if_alloc! {
     }
 
     impl<'a> ctx::TryFromCtx<'a, RelocCtx> for Reloc {
-        type Error = ::error::Error;
-        type Size = usize;
-        fn try_from_ctx(bytes: &'a [u8], (is_rela, Ctx { container, le }): RelocCtx) -> result::Result<(Self, Self::Size), Self::Error> {
+        type Error = crate::error::Error;
+        fn try_from_ctx(bytes: &'a [u8], (is_rela, Ctx { container, le }): RelocCtx) -> result::Result<(Self, usize), Self::Error> {
             use scroll::Pread;
             let reloc = match container {
                 Container::Little => {
@@ -323,11 +343,9 @@ if_alloc! {
     }
 
     impl ctx::TryIntoCtx<RelocCtx> for Reloc {
-        type Error = ::error::Error;
-        type Size = usize;
-        // TODO: I think this is a bad idea
+        type Error = crate::error::Error;
         /// Writes the relocation into `bytes`
-        fn try_into_ctx(self, bytes: &mut [u8], (is_rela, Ctx {container, le}): RelocCtx) -> result::Result<Self::Size, Self::Error> {
+        fn try_into_ctx(self, bytes: &mut [u8], (is_rela, Ctx {container, le}): RelocCtx) -> result::Result<usize, Self::Error> {
             use scroll::Pwrite;
             match container {
                 Container::Little => {
@@ -362,22 +380,12 @@ if_alloc! {
 
     impl fmt::Debug for Reloc {
         fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-            if let Some(addend) = self.r_addend {
-                write!(f,
-                    "r_offset: {:x} r_typ: {} r_sym: {} r_addend: {:x}",
-                    self.r_offset,
-                    self.r_type,
-                    self.r_sym,
-                    addend,
-                )
-            } else {
-                write!(f,
-                    "r_offset: {:x} r_typ: {} r_sym: {}",
-                    self.r_offset,
-                    self.r_type,
-                    self.r_sym,
-                )
-            }
+            f.debug_struct("Reloc")
+                .field("r_offset", &format_args!("{:x}", self.r_offset))
+                .field("r_addend", &format_args!("{:x}", self.r_addend.unwrap_or(0)))
+                .field("r_sym", &self.r_sym)
+                .field("r_type", &self.r_type)
+                .finish()
         }
     }
 
@@ -406,7 +414,7 @@ if_alloc! {
     impl<'a> RelocSection<'a> {
         #[cfg(feature = "endian_fd")]
         /// Parse a REL or RELA section of size `filesz` from `offset`.
-        pub fn parse(bytes: &'a [u8], offset: usize, filesz: usize, is_rela: bool, ctx: Ctx) -> ::error::Result<RelocSection<'a>> {
+        pub fn parse(bytes: &'a [u8], offset: usize, filesz: usize, is_rela: bool, ctx: Ctx) -> crate::error::Result<RelocSection<'a>> {
             // TODO: better error message when too large (see symtab implementation)
             let bytes = bytes.pread_with(offset, filesz)?;
 
@@ -430,10 +438,17 @@ if_alloc! {
         }
 
         /// The number of relocations in the section.
+        #[inline]
         pub fn len(&self) -> usize {
             self.count
         }
 
+        /// Returns true if section has no relocations.
+        #[inline]
+        pub fn is_empty(&self) -> bool {
+            self.count == 0
+        }
+
         /// Iterate over all relocations.
         pub fn iter(&self) -> RelocIterator<'a> {
             self.into_iter()
@@ -469,6 +484,18 @@ if_alloc! {
         ctx: RelocCtx,
     }
 
+    impl<'a> fmt::Debug for RelocIterator<'a> {
+        fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
+            fmt.debug_struct("RelocIterator")
+                .field("bytes", &"<... redacted ...>")
+                .field("offset", &self.offset)
+                .field("index", &self.index)
+                .field("count", &self.count)
+                .field("ctx", &self.ctx)
+                .finish()
+        }
+    }
+
     impl<'a> Iterator for RelocIterator<'a> {
         type Item = Reloc;
 
