@@ -257,7 +257,7 @@ pub fn shf_to_str(shf: u32) -> &'static str {
         SHF_COMPRESSED => "SHF_COMPRESSED",
         //SHF_MASKOS..SHF_MASKPROC => "SHF_OSFLAG",
         SHF_ORDERED => "SHF_ORDERED",
-        _ => "SHF_UNKNOWN"
+        _ => "SHF_UNKNOWN",
     }
 }
 
@@ -276,7 +276,7 @@ macro_rules! elf_section_header_std_impl { ($size:ty) => {
         use crate::elf::section_header::SectionHeader as ElfSectionHeader;
 
         use plain::Plain;
-        use crate::alloc::vec::Vec;
+        use alloc::vec::Vec;
 
         if_std! {
             use crate::error::Result;
@@ -341,7 +341,6 @@ macro_rules! elf_section_header_std_impl { ($size:ty) => {
     } // end if_alloc
 };}
 
-
 pub mod section_header32 {
     pub use crate::elf::section_header::*;
 
@@ -352,7 +351,6 @@ pub mod section_header32 {
     elf_section_header_std_impl!(u32);
 }
 
-
 pub mod section_header64 {
 
     pub use crate::elf::section_header::*;
@@ -377,7 +375,7 @@ if_alloc! {
     use crate::container::{Container, Ctx};
 
     #[cfg(feature = "endian_fd")]
-    use crate::alloc::vec::Vec;
+    use alloc::vec::Vec;
 
     #[derive(Default, PartialEq, Clone)]
     /// A unified SectionHeader - convertable to and from 32-bit and 64-bit variants
@@ -435,16 +433,22 @@ if_alloc! {
         }
         /// Parse `count` section headers from `bytes` at `offset`, using the given `ctx`
         #[cfg(feature = "endian_fd")]
-        pub fn parse(bytes: &[u8], mut offset: usize, count: usize, ctx: Ctx) -> error::Result<Vec<SectionHeader>> {
+        pub fn parse(bytes: &[u8], mut offset: usize, mut count: usize, ctx: Ctx) -> error::Result<Vec<SectionHeader>> {
             use scroll::Pread;
-            let mut section_headers = Vec::with_capacity(count);
-            let mut nsection_headers = count;
+            // Zero offset means no section headers, not even the null section header.
+            if offset == 0 {
+                return Ok(Vec::new());
+            }
             let empty_sh = bytes.gread_with::<SectionHeader>(&mut offset, ctx)?;
             if count == 0 as usize {
-                nsection_headers = empty_sh.sh_size as usize;
+                // Zero count means either no section headers if offset is also zero (checked
+                // above), or the number of section headers overflows SHN_LORESERVE, in which
+                // case the count is stored in the sh_size field of the null section header.
+                count = empty_sh.sh_size as usize;
             }
+            let mut section_headers = Vec::with_capacity(count);
             section_headers.push(empty_sh);
-            for _ in 1..nsection_headers {
+            for _ in 1..count {
                 let shdr = bytes.gread_with(&mut offset, ctx)?;
                 section_headers.push(shdr);
             }
